int batteryCapacity = 2500;      // значение емкости батареи в мАч
float resistance = 10.0;         // измеренное сопротивление мощного резистора
int cutoffVoltage = 1600;        // максимальное напряжение на батарее (в мВ), которое не должно быть превышено
float cutoffTemperatureC = 35;   // максимальная температура батареи, которая не должна быть превышена (в градусах C)
//float cutoffTemperatureF = 95; // максимальная температура батареи, которая не должна быть превышена (в градусах F)
long cutoffTime = 46800000;      // максимальное время заряда в 13 часов, которое не должно быть превышено

int outputPin = 9;         // провод выходного сигнала подключен к цифровому выводу 9
int outputValue = 150;     // значение выходного ШИМ сигнала 

int analogPinOne = 0;      // первый датчик напряжения подключен к аналоговому выводу 0
float valueProbeOne = 0;   // переменная для хранения значения на analogPinOne
float voltageProbeOne = 0; // рассчитанное напряжение на analogPinOne

int analogPinTwo = 1;      // второй датчик напряжения подключен к аналоговому выводу 1
float valueProbeTwo = 0;   // переменная для хранения значения на analogPinTwo
float voltageProbeTwo = 0; // рассчитанное напряжение на analogPinTwo

int analogPinThree = 2;     // третий датчик напряжения подключен к аналоговому выводу 2
float valueProbeThree = 0;  // переменная для хранения значения на analogPinThree
float tmp36Voltage = 0;     // рассчитанное напряжение на analogPinThree
float temperatureC = 0;     // рассчитанная температура датчика в градусах C
//float temperatureF = 0;   // рассчитанная температура датчика в градусах F

float voltageDifference = 0;                   // разница между напряжениями на analogPinOne и analogPinTwo
float batteryVoltage = 0;                      // рассчитанное напряжение на батарее
float current = 0;                             // рассчитанный ток, протекающий через нагрузку в (мА)
float targetCurrent = batteryCapacity / 10;    // целевой выходной ток (в мА) устанавливается в значение 
                                               // C/10 или 1/10 от емкости батареи
float currentError = 0;         // разница между целевым и фактическим токами (в мА)



void setup()
{
  Serial.begin(9600);             // настройка последовательного интерфейса
  pinMode(outputPin, OUTPUT);     // установить вывод, как выход
}



void loop()
{
    
  analogWrite(outputPin, outputValue);  // записать выходное значение в выходной вывод

  Serial.print("Output: ");     // показать выходные значения для контроля на компьютере
  Serial.println(outputValue); 

  valueProbeOne = analogRead(analogPinOne);     // считать входное значение на первом пробнике
  voltageProbeOne = (valueProbeOne*5000)/1023;  // рассчитать напряжение на первом пробнике в милливольтах
  Serial.print("Voltage Probe One (mV): ");     // показать напряжение на первом пробнике
  Serial.println(voltageProbeOne);  
  
  valueProbeTwo = analogRead(analogPinTwo);    // считать входное значение на втором пробнике
  voltageProbeTwo = (valueProbeTwo*5000)/1023; // рассчитать напряжение на втором пробнике в милливольтах
  Serial.print("Voltage Probe Two (mV): ");    // показать напряжение на втором пробнике
  Serial.println(voltageProbeTwo);  
  
  batteryVoltage = 5000 - voltageProbeTwo;     // рассчитать напряжение на батарее
  Serial.print("Battery Voltage (mV): ");      // показать напряжение на батарее
  Serial.println(batteryVoltage); 

  current = (voltageProbeTwo - voltageProbeOne) / resistance;     // рассчитать ток заряда
  Serial.print("Target Current (mA): ");                          // показать целевой ток 
  Serial.println(targetCurrent);  
  Serial.print("Battery Current (mA): ");                         // показать фактический ток
  Serial.println(current);  
      
  currentError = targetCurrent - current;    // разница между целевым и измеренным токами
  Serial.print("Current Error  (mA): ");     // показать ошибку установки тока 
  Serial.println(currentError);     

  valueProbeThree = analogRead(analogPinThree);  // считать входное значение третьего пробника,  
  tmp36Voltage = valueProbeThree * 5.0;          // преобразуя его в напряжение
  tmp36Voltage /= 1024.0; 
 
  temperatureC = (tmp36Voltage - 0.5) * 100 ;     // преобразование, исходя из зависимости в 10 мВ на градус со сдвиком в 500 мВ
                                                  // ((напряжение - 500 мВ) умножить на 100)
  Serial.print("Temperature (degrees C) ");       // показать температуру в градусах Цельсия
  Serial.println(temperatureC); 
 
 /*
  temperatureF = (temperatureC * 9.0 / 5.0) + 32.0;     //преобразовать в градусы Фаренгейта
  Serial.print("Temperature (degrees F) ");
  Serial.println(temperatureF); 
 */
 
  Serial.println();     // дополнительные пустые строки, чтобы облегчить чтение данных при отладке
  Serial.println();  



  if(abs(currentError) > 10)     // если ошибка установки тока достаточно велика, то подстроить выходное напряжение
   {
    outputValue = outputValue + currentError / 10;

    if(outputValue < 1)    // выходное значение никогда не может быть ниже 0
     {
      outputValue = 0;
     }

    if(outputValue > 254)  // выходное значение никогда не может быть выше 255
     {
      outputValue = 255;
     }
    
    analogWrite(outputPin, outputValue);     // записать новое выходное значение
   }
 
 
  if(temperatureC > cutoffTemperatureC)     // остановить зарядку, если температура батареи превысила безопасный порог
   {
    outputValue = 0;
    Serial.print("Max Temperature Exceeded");
   }
   
  /*
  if(temperatureF > cutoffTemperatureF)     // остановить зарядку, если температура батареи превысила безопасный порог
   {
    outputValue = 0;
   }
   */
   
   if(batteryVoltage > cutoffVoltage)       // остановить зарядку, если напряжение на батарее превысило безопасный порог
   {
    outputValue = 0;
    Serial.print("Max Voltage Exceeded");
   }  
 
   if(millis() > cutoffTime)                // остановить зарядку, если время заряда превысило порог
   {
    outputValue = 0;
    Serial.print("Max Charge Time Exceeded");
   }  

   delay(10000);     // задержка в 10 секунд перед следующей итерацией цикла
}




